<?php
/*
 * ARI Orbit Slider
 *
 * @package		ARI Orbit Slider
 * @version		1.0.0
 * @author		ARI Soft
 * @copyright	Copyright (c) 2010 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

defined('ARI_FRAMEWORK_LOADED') or die('Direct Access to this location is not allowed.');

jimport('joomla.filter.filterinput');

AriKernel::import('Utils.Utils');
AriKernel::import('Utils.AppUtils');
AriKernel::import('Web.JSON.JSONHelper');
AriKernel::import('FileSystem.Folder');

class AriOrbitSliderHelper
{
	function includeAssets($params)
	{
		static $loaded;

		if ($loaded)
			return ;
			
		$loadJQuery = (bool)$params->get('includeJQuery', true);
		$noConflict = (bool)$params->get('noConflict', true);
		$theme = $params->get('theme', '');

		$doc = JFactory::getDocument();
		$baseUri = JURI::root(true) . '/modules/mod_ariorbitslider/mod_ariorbitslider/js/';
		if ($loadJQuery) 
		{
            if (J3_0)
                JHtml::_('jquery.framework', $noConflict);
            else
            {
			    $doc->addScript('//ajax.googleapis.com/ajax/libs/jquery/1.12.2/jquery.min.js');

			    if ($noConflict)
				    $doc->addScript($baseUri . 'jquery.noconflict.js');
            }
		}

		$doc->addScript($baseUri . 'jquery.orbit.min.js');

		$theme = JFilterInput::getInstance()->clean($theme, 'WORD');
		if (empty($theme)) $theme = 'default';

		$doc->addStyleSheet($baseUri . 'themes/' . $theme . '/style.css');
		if (@file_exists(JPATH_ROOT . DS . 'modules' . DS . 'mod_ariorbitslider' . DS . 'mod_ariorbitslider' . DS . 'js' . DS . 'themes' . DS . $theme . DS . 'style.ie.css'))
		{
			$doc->addCustomTag(
				sprintf('<!--[if IE]><link rel="stylesheet" href="%s" type="text/css" /><![endif]-->',
					$baseUri . 'themes/' . $theme . '/style.ie.css')
			);
		}

		$loaded = true;
	}
	
	function initSlider($id, $params)
	{
		AriOrbitSliderHelper::includeAssets($params);

		$clientParams = AriOrbitSliderHelper::getClientParams($params);
		$doc = JFactory::getDocument();

		$doc->addScriptDeclaration(
			sprintf('jQuery(window).load(function() { var $ = window.jQueryOrbitSlider || jQuery; $("#%1$s").orbit(%2$s); });',
				$id,
				$clientParams ? AriJSONHelper::encode($clientParams) : '')
		);
		
		$captionSize = max(intval($params->get('style_caption_fontSize', 12), 10), 1);
		$captionWeight = $params->get('style_caption_fontWeight', 'normal');
		$captionColor = $params->get('style_caption_color');
		$captionBgColor = $params->get('style_caption_bgColor');
		$captionOpacity = min(max(floatval($params->get('style_caption_opacity', 0.6)), 0), 1);
		$showLoadingPane = (bool)$params->get('showLoadingPane');
		
		$styleDec = sprintf('#%1$s div.caption p {font-size:%2$dpx;font-weight:%6$s}#%1$s div.caption {color:%3$s;background-color:%4$s;opacity:%5$f}',
			$id,
			$captionSize,
			$captionColor,
			$captionBgColor,
			$captionOpacity,
			$captionWeight);
			
		$initWidth = intval($params->get('initWidth'), 10);
		$initHeight = intval($params->get('initHeight'), 10);
		if ($showLoadingPane)
		{
			if ($initWidth && $initHeight)
				$styleDec .= sprintf('#%1$s.ari-orbit-slider-loading{width:%2$dpx;height:%3$dpx;}',
					$id,
					$initWidth,
					$initHeight);
		}
		else 
		{
			if ($initWidth && $initHeight)
				$styleDec .= sprintf('#%1$s.orbit{width:%2$dpx;height:%3$dpx;}',
					$id,
					$initWidth,
					$initHeight);
		}
			
		if ($params->get('customstyle'))
		{
			$extraStyles = trim($params->get('customstyle'));
			$extraStyles = str_replace('{$id}', '#' . $id, $extraStyles);
			if (!empty($extraStyles))
				$styleDec .= $extraStyles;
		}
		
		$doc->addStyleDeclaration($styleDec);
		$doc->addCustomTag(
			sprintf('<!--[if IE]><style type="text/css">#%1$s div.caption {filter:progid:DXImageTransform.Microsoft.Alpha(opacity=%2$d)}</style><![endif]-->',
				$id,
				ceil($captionOpacity * 100))
		);
	}
	
	function getClientParams($params)
	{
		$clientParams = array(
			'animation' => 'fade',
			'animationSpeed' => 800,
			'advanceSpeed' => 4000,
			'startClockOnMouseOut' => true,
			'startClockOnMouseOutAfter' => 3000,
			'directionalNav' => true,
			'captions' => true,
			'captionAnimationSpeed' => 800,
			'timer' => false,
			'bullets' => false,
			'pauseOnHover' => false
		);
		
		$sliderParams = array();
		foreach ($clientParams as $key => $value)
		{
			$param = $params->get('opt_' . $key, null);
			if (is_null($param))
				continue ;
				
			$param = AriUtils::parseValueBySample($param, $value);
			if ($value !== $param)
				$sliderParams[$key] = $param;
		}
		
		if ($params->get('opt_timer') && $params->get('opt_timer') == 'auto')
			$sliderParams['timer'] = 'auto';
		
		return $sliderParams;
	}
	
	function getSlides($params)
	{
		$slides = array();

		$pathList = AriOrbitSliderHelper::getPathList($params->get('path'));
		if (count($pathList) == 0)
			return $slides;

		$extraFields = array();
		$i18n = (bool)$params->get('i18n', false);
		$descrFile = trim($params->get('descrFile', ''));
		$processDescrFile = !empty($descrFile);
		$processSubDir = (bool)$params->get('subdir');
		$images = array();
		$sort = AriOrbitSliderHelper::getSortExpr($params);
		foreach ($pathList as $path)
		{
			if (empty($path))
				continue ;
			
			if (@is_file($path))
			{
				$images[] = $path;
				$path = dirname($path);
			}
			else
			{
				$folderImages = AriFolder::files(
					$path,
					'.(jpg|gif|jpeg|png|bmp|JPG|GIF|JPEG|BMP|PNG)$', 
					$processSubDir,
					true,
					$sort);
	
				if (is_array($folderImages) && count($folderImages) > 0)
				{				
					$images = array_merge($images, $folderImages);
				}
			}
			
			if ($processDescrFile)
			{
				$folderExtraFields = AriAppUtils::getExtraFieldsFromINI($path, $descrFile, $processSubDir, true, $i18n);
				if (is_array($folderExtraFields) && count($folderExtraFields) > 0)
				{				
					$extraFields = array_merge($extraFields, $folderExtraFields);
				}
			}
		}
		
		$useThumbs = (bool)$params->get('imglist_useThumbs');
		$cachePath = $useThumbs ? AriOrbitSliderHelper::getCachePath() : null;
		$thumbPath = $params->get('imglist_thumbPath');
		$thumbWidth = intval($params->get('imglist_thumbWidth'), 10);
		$thumbHeight = intval($params->get('imglist_thumbHeight'), 10);
		foreach ($images as $image)
		{
			$slide = array(
				'src' => $image,
				'image' => str_replace('\\', '/', $image)
			);
			
			if ($processDescrFile && isset($extraFields[$image]))
				$slide = array_merge($extraFields[$image], $slide);
				
			if ($useThumbs)
				$slide = AriOrbitSliderHelper::generateThumbnail($slide, $thumbWidth, $thumbHeight, $thumbPath, $cachePath);
			
			$slides[] = $slide;
		}

		return $slides;
	}

	function prepareSlides($slides, $params)
	{
		$newSlides = array();
		
		$target = $params->get('linkTarget', '_self');
		$baseUri = JURI::base(true);
		$lightboxEngine = AriOrbitSliderHelper::getLightboxEngine($params);
		$lightboxGroup = uniqid('cc');
		foreach ($slides as $slide)
		{
			$slideLink = !empty($slide['link']) ? $slide['link'] : '';
		
			$isLink = !empty($slideLink);
			$description = isset($slide['description']) ? $slide['description'] : '';
		
			$lnkAttrs = null;
			$imgAttrs = array('src' => $baseUri . '/' . $slide['image'], 'alt' => '', 'title' => $description, 'class' => 'orbitslider-item');
			
			if (empty($slide['width']) && empty($slide['height']))
			{
				$size = @getimagesize(JPATH_ROOT . DS . $slide['image']);
				if (!empty($size))
				{
					$slide['width'] = $size[0];
					$slide['height'] = $size[1];
				}
			}
			
			if (!empty($slide['width'])) $imgAttrs['width'] = $slide['width'];
			if (!empty($slide['height'])) $imgAttrs['height'] = $slide['height'];
			if ($isLink)
			{
				$lnkAttrs = array('href' => $slide['link'], 'target' => $target);
				if ($description)
					$lnkAttrs['title'] = $description;
					
				$skip_lb = false;
				if (!is_null($lightboxEngine) && strpos($slideLink, 'skip_lb') !== false)
				{
					$uri = new JURI($slideLink);
					$skip_lb = (bool)$uri->getVar('skip_lb');
					$uri->delVar('skip_lb');
					$slideLink = $uri->toString();
					$lnkAttrs['href'] = $slideLink;
				}
				
				if (!$skip_lb && !is_null($lightboxEngine))
					list($lnkAttrs, $imgAttrs) = $lightboxEngine->modifyAttrs($lnkAttrs, $imgAttrs, $lightboxGroup, $params);
				else
				{
					$originalLink = $slideLink;
					if (strpos($originalLink, '_target') !== false)
					{
						$uri = new JURI($originalLink);
						$linkTarget = $uri->getVar('_target');
						if (!is_null($linkTarget))
						{
							$uri->delVar('_target');
							$lnkAttrs['target'] = $linkTarget;
							$lnkAttrs['href'] = $uri->toString();
						}
					}
				}
			}
					
			$slide['lnkAttrs'] = $lnkAttrs;
			$slide['imgAttrs'] = $imgAttrs;
			$newSlides[] = $slide;
		}
		
		return $newSlides;
	}

	function generateThumbnail($slide, $thumbWidth, $thumbHeight, $thumbPath, $cachePath)
	{
		$img = $slide['src'];
		$imgUri = $slide['image'];
		$thumbFile = null;
		
		if ($thumbPath)
		{
			$pathInfo = pathinfo($img);
			$thumbImg = $pathInfo['dirname'] . DS .  str_replace('{$fileName}', $pathInfo['basename'], $thumbPath);
			if (@file_exists(JPATH_ROOT . DS . $thumbImg) && @is_file(JPATH_ROOT . DS . $thumbImg))
			{
				$thumbFile = $thumbImg;
			}
		}

		if (is_null($thumbFile))
		{
			$thumbName = AriImageHelper::generateThumbnail(
				JPATH_ROOT . DS . $img, 
				JPATH_ROOT . DS . $cachePath, 
				'ais',
				$thumbWidth,
				$thumbHeight);
			if ($thumbName)
			{
				$thumbFile = $cachePath . DS . $thumbName;
			}
			else
			{
				$thumbFile = $thumbImg;
			}
		}

		$slide['src'] = $thumbFile;
		$slide['image'] = str_replace('\\', '/', $thumbFile);
		if (empty($slide['link']))
			$slide['link'] = $imgUri;
			
		$thumbSize = AriImageHelper::getThumbnailDimension(JPATH_ROOT . DS . $thumbFile, $thumbWidth, $thumbHeight);
		if (!empty($thumbSize['w'])) $slide['width'] = $thumbSize['w'];
		if (!empty($thumbSize['h'])) $slide['height'] = $thumbSize['h'];
		
		return $slide;
	}

	function getCachePath()
	{
		$cacheDir = 'images';
		$extCacheDir = $cacheDir . DS . 'ariorbitslider';
		if (@file_exists($extCacheDir) && is_dir($extCacheDir))
		{
			$cacheDir = $extCacheDir;
		}
		
		return $cacheDir;
	}
	
	function getSortExpr($params)
	{
		$sortBy = $params->get('sortBy');
		if (empty($sortBy) || !in_array($sortBy, array('filename', 'modified', 'random')))
			return null;
			
		return array(
			'sortBy' => $sortBy, 
			'sortDir' => ($params->get('sortDir') == 'desc' ? 'desc' : 'asc')
		);
	}
	
	function getPathList($path)
	{
		$pathList = array();
		if (empty($path))
			return $pathList;

		$pathList = explode("\n", $path);
		array_walk($pathList, array('AriFolder', 'clean'));
		$pathList = array_unique($pathList);
		
		return $pathList;
	}

	function getLightboxEngine($params)
	{
		$engine = null;
		$engineName = ucfirst(JFilterInput::getInstance()->clean($params->get('lightboxEngine'), 'WORD'));
		if (empty($engineName))
			return null;
		
		AriKernel::import('OrbitSlider.Lightbox.' . $engineName);
		
		$className = $engineName . 'OrbitSliderEngine';
		if (class_exists($className))
		{
			$engine = new $className();
			if (!$engine->preCheck())
			{
				$engine = null;
			}
		}
		
		return $engine;
	}
}